cheesecake.subprocess

Package cheesecake :: Module subprocess |
[show private | hide private]
[frames | no frames]

## Module cheesecake.subprocess

    subprocess - Subprocesses with accessible I/O streams

    This module allows you to spawn processes, connect to their
    input/output/error pipes, and obtain their return codes.  This module
    intends to replace several other, older modules and functions, like:

    os.system
    os.spawn*
    os.popen*
    popen2.*
    commands.*

    Information about how the subprocess module can be used to replace these
    modules and functions can be found below.

    Using the subprocess module
    ===========================
    This module defines one class called Popen:

    class Popen(args, bufsize=0, executable=None,
                stdin=None, stdout=None, stderr=None,
                preexec_fn=None, close_fds=False, shell=False,
                cwd=None, env=None, universal_newlines=False,
                startupinfo=None, creationflags=0):

    Arguments are:

    args should be a string, or a sequence of program arguments.  The
    program to execute is normally the first item in the args sequence or
    string, but can be explicitly set by using the executable argument.

    On UNIX, with shell=False (default): In this case, the Popen class
    uses os.execvp() to execute the child program.  args should normally
    be a sequence.  A string will be treated as a sequence with the string
    as the only item (the program to execute).

    On UNIX, with shell=True: If args is a string, it specifies the
    command string to execute through the shell.  If args is a sequence,
    the first item specifies the command string, and any additional items
    will be treated as additional shell arguments.

    On Windows: the Popen class uses CreateProcess() to execute the child
    program, which operates on strings.  If args is a sequence, it will be
    converted to a string using the list2cmdline method.  Please note that
    not all MS Windows applications interpret the command line the same
    way: The list2cmdline is designed for applications using the same
    rules as the MS C runtime.

    bufsize, if given, has the same meaning as the corresponding argument
    to the built-in open() function: 0 means unbuffered, 1 means line
    buffered, any other positive value means use a buffer of
    (approximately) that size.  A negative bufsize means to use the system
    default, which usually means fully buffered.  The default value for
    bufsize is 0 (unbuffered).

    stdin, stdout and stderr specify the executed programs' standard
    input, standard output and standard error file handles, respectively.
    Valid values are PIPE, an existing file descriptor (a positive
    integer), an existing file object, and None.  PIPE indicates that a
    new pipe to the child should be created.  With None, no redirection
    will occur; the child's file handles will be inherited from the
    parent.  Additionally, stderr can be STDOUT, which indicates that the
    stderr data from the applications should be captured into the same
    file handle as for stdout.

    If preexec_fn is set to a callable object, this object will be called
    in the child process just before the child is executed.

    If close_fds is true, all file descriptors except 0, 1 and 2 will be
    closed before the child process is executed.

    if shell is true, the specified command will be executed through the
    shell.

    If cwd is not None, the current directory will be changed to cwd
    before the child is executed.

    If env is not None, it defines the environment variables for the new
    process.

    If universal_newlines is true, the file objects stdout and stderr are
    opened as a text files, but lines may be terminated by any of '\n',
    the Unix end-of-line convention, '\r', the Macintosh convention or
    '\r\n', the Windows convention.  All of these external representations
    are seen as '\n' by the Python program.  Note: This feature is only
    available if Python is built with universal newline support (the
    default).  Also, the newlines attribute of the file objects stdout,
    stdin and stderr are not updated by the communicate() method.

    The startupinfo and creationflags, if given, will be passed to the
    underlying CreateProcess() function.  They can specify things such as
    appearance of the main window and priority for the new process.
    (Windows only)

    This module also defines two shortcut functions:

    call(*args, **kwargs):
        Run command with arguments.  Wait for command to complete, then
        return the returncode attribute.

        The arguments are the same as for the Popen constructor.  Example:

        retcode = call(["ls", "-l"])

    Exceptions
    ----------
    Exceptions raised in the child process, before the new program has
    started to execute, will be re-raised in the parent.  Additionally,
    the exception object will have one extra attribute called
    'child_traceback', which is a string containing traceback information
    from the childs point of view.

    The most common exception raised is OSError.  This occurs, for
    example, when trying to execute a non-existent file.  Applications
    should prepare for OSErrors.

    A ValueError will be raised if Popen is called with invalid arguments.

    Security
    --------
    Unlike some other popen functions, this implementation will never call
    /bin/sh implicitly.  This means that all characters, including shell
    metacharacters, can safely be passed to child processes.

    Popen objects
    =============
    Instances of the Popen class have the following methods:

    poll()
        Check if child process has terminated.  Returns returncode
        attribute.

    wait()
        Wait for child process to terminate.  Returns returncode attribute.

    communicate(input=None)
        Interact with process: Send data to stdin.  Read data from stdout
        and stderr, until end-of-file is reached.  Wait for process to
        terminate.  The optional stdin argument should be a string to be
        sent to the child process, or None, if no data should be sent to
        the child.

        communicate() returns a tuple (stdout, stderr).

        Note: The data read is buffered in memory, so do not use this
        method if the data size is large or unlimited.

    The following attributes are also available:

    stdin
        If the stdin argument is PIPE, this attribute is a file object
        that provides input to the child process.  Otherwise, it is None.

    stdout
        If the stdout argument is PIPE, this attribute is a file object
        that provides output from the child process.  Otherwise, it is
        None.

    stderr
        If the stderr argument is PIPE, this attribute is file object that
        provides error output from the child process.  Otherwise, it is
        None.

    pid
        The process ID of the child process.

    returncode
        The child return code.  A None value indicates that the process
        hasn't terminated yet.  A negative value -N indicates that the
        child was terminated by signal N (UNIX only).

    Replacing older functions with the subprocess module
    ====================================================
    In this section, "a ==> b" means that b can be used as a replacement
    for a.

    Note: All functions in this section fail (more or less) silently if
    the executed program cannot be found; this module raises an OSError
    exception.

    In the following examples, we assume that the subprocess module is
    imported with "from subprocess import *".

    Replacing /bin/sh shell backquote
    ---------------------------------
    output=`mycmd myarg`
    ==>
    output = Popen(["mycmd", "myarg"], stdout=PIPE).communicate()[0]

    Replacing shell pipe line
    -------------------------
    output=`dmesg | grep hda`
    ==>
    p1 = Popen(["dmesg"], stdout=PIPE)
    p2 = Popen(["grep", "hda"], stdin=p1.stdout)
    output = p2.communicate()[0]

    Replacing os.system()
    ---------------------
    sts = os.system("mycmd" + " myarg")
    ==>
    p = Popen("mycmd" + " myarg", shell=True)
    sts = os.waitpid(p.pid, 0)

    Note:

    * Calling the program through the shell is usually not required.

    * It's easier to look at the returncode attribute than the
      exitstatus.

    A more real-world example would look like this:

    try:
        retcode = call("mycmd" + " myarg", shell=True)
        if retcode < 0:
            print >>sys.stderr, "Child was terminated by signal", -retcode
        else:
            print >>sys.stderr, "Child returned", retcode
    except OSError, e:
        print >>sys.stderr, "Execution failed:", e

    Replacing os.spawn*
    -------------------
    P_NOWAIT example:

    pid = os.spawnlp(os.P_NOWAIT, "/bin/mycmd", "mycmd", "myarg")
    ==>
    pid = Popen(["/bin/mycmd", "myarg"]).pid

    P_WAIT example:

    retcode = os.spawnlp(os.P_WAIT, "/bin/mycmd", "mycmd", "myarg")
    ==>
    retcode = call(["/bin/mycmd", "myarg"])

    Vector example:

    os.spawnvp(os.P_NOWAIT, path, args)
    ==>
    Popen([path] + args[1:])

    Environment example:

    os.spawnlpe(os.P_NOWAIT, "/bin/mycmd", "mycmd", "myarg", env)
    ==>
    Popen(["/bin/mycmd", "myarg"], env={"PATH": "/usr/bin"})

    Replacing os.popen*
    -------------------
    pipe = os.popen(cmd, mode='r', bufsize)
    ==>
    pipe = Popen(cmd, shell=True, bufsize=bufsize, stdout=PIPE).stdout

    pipe = os.popen(cmd, mode='w', bufsize)
    ==>
    pipe = Popen(cmd, shell=True, bufsize=bufsize, stdin=PIPE).stdin

    (child_stdin, child_stdout) = os.popen2(cmd, mode, bufsize)
    ==>
    p = Popen(cmd, shell=True, bufsize=bufsize,
              stdin=PIPE, stdout=PIPE, close_fds=True)
    (child_stdin, child_stdout) = (p.stdin, p.stdout)

    (child_stdin,
     child_stdout,
     child_stderr) = os.popen3(cmd, mode, bufsize)
    ==>
    p = Popen(cmd, shell=True, bufsize=bufsize,
              stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True)
    (child_stdin,
     child_stdout,
     child_stderr) = (p.stdin, p.stdout, p.stderr)

    (child_stdin, child_stdout_and_stderr) = os.popen4(cmd, mode, bufsize)
    ==>
    p = Popen(cmd, shell=True, bufsize=bufsize,
              stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
    (child_stdin, child_stdout_and_stderr) = (p.stdin, p.stdout)

    Replacing popen2.*
    ------------------
    Note: If the cmd argument to popen2 functions is a string, the command
    is executed through /bin/sh.  If it is a list, the command is directly
    executed.

    (child_stdout, child_stdin) = popen2.popen2("somestring", bufsize, mode)
    ==>
    p = Popen(["somestring"], shell=True, bufsize=bufsize
              stdin=PIPE, stdout=PIPE, close_fds=True)
    (child_stdout, child_stdin) = (p.stdout, p.stdin)

    (child_stdout, child_stdin) = popen2.popen2(["mycmd", "myarg"], bufsize, mode)
    ==>
    p = Popen(["mycmd", "myarg"], bufsize=bufsize,
              stdin=PIPE, stdout=PIPE, close_fds=True)
    (child_stdout, child_stdin) = (p.stdout, p.stdin)

    The popen2.Popen3 and popen3.Popen4 basically works as subprocess.Popen,
    except that:

    * subprocess.Popen raises an exception if the execution fails
    * the capturestderr argument is replaced with the stderr argument.
    * stdin=PIPE and stdout=PIPE must be specified.
    * popen2 closes all filedescriptors by default, but you have to specify
      close_fds=True with subprocess.Popen.

Classes
`Popen` |

Exceptions
`ProcessError` | This exception is raised when there is an error calling a subprocess.

Function Summary
| ``call`(*args, **kwargs)` Run command with arguments.

Variable Summary
`int` | `PIPE` = -1
`int` | `STDOUT` = -2

Function Details

### call(*args, **kwargs) Run command with arguments. Wait for command to complete, then return the returncode attribute. The arguments are the same as for the Popen constructor. Example: retcode = call(["ls", "-l"])

Variable Details

    ### PIPE Type: `int` Value:
        -1

    ### STDOUT Type: `int` Value:
        -2

Generated by Epydoc 2.1 on Fri Feb 9 02:15:12 2007 | http://epydoc.sf.net